/* setfsb.c - FSB modification tool
 * Copyright (C) 2003 Georg Lukas <georg@boerde.de>
 * (This Program is under the GNU General Public License)
 *
 * WARNING!!! DO NOT USE THIS PROGRAM!!!
 *
 * If you use it despite the warning, it might ruin your CPU, motherboard or
 * your sex life!
 *
 * Ok, so what does it do? It changes the FSB and PCI clocks on motherboards
 * equipped with the W83194R-39A PLL. This allows you to overclock and crash
 * your machine. Pretty cool, eh? ;)
 *
 * You need a working IC/SMBus (see the lm_sensors package for drivers) to be
 * able to access the W83194R-39A, this program uses the i2c-dev interface for
 * communication with it.
 *
 * I PROVIDE NO WARRANTY WHATSOEVER. IF YOU FSCK UP YOUR MAINBOARD, IT IS ALL
 * YOUR FAULT. YOU WERE WARNED!
 */

#define VERSION "setfsb 0.0.02 (C) 2003-02-17 Georg Lukas"

#define SMB_QUERY_ONLY -1
#define SMB_RESET 0

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/i2c.h>
#include <linux/i2c-dev.h>

struct fsb_mapping {
	int fsb;
	int pci;
	unsigned char reg;
};

struct fsb_mapping w83194r[] = {
	/*{ 124, 41, 0x00 },
	{  75, 38, 0x01 },
	{  83, 41, 0x02 },
	{  66, 33, 0x03 },
	{ 103, 34, 0x04 },
	{ 112, 37, 0x05 },
	{ 134, 44, 0x06 },
	{ 100, 33, 0x07 },

	{ 120, 40, 0x08 },
	{ 115, 38, 0x09 },
	{ 110, 37, 0x0a },
	{ 105, 35, 0x0b },
	{ 140, 35, 0x0c },
	{ 150, 38, 0x0d },
	{ 123, 31, 0x0e },
	{ 133, 33, 0x0f },
	{ 0, }
};*/
	{  66, 33, 0x03 },
	{  75, 38, 0x01 },
	{  83, 41, 0x02 },
	{ 100, 33, 0x07 },
	{ 103, 34, 0x04 },
	{ 105, 35, 0x0b },
	{ 110, 37, 0x0a },
	{ 112, 37, 0x05 },
	{ 115, 38, 0x09 },
	{ 120, 40, 0x08 },
	{ 123, 31, 0x0e },
	{ 124, 41, 0x00 },
	{ 133, 33, 0x0f },
	{ 134, 44, 0x06 },
	{ 140, 35, 0x0c },
	{ 150, 38, 0x0d },
	{ 0, }
};


void usage(char *progname) {
	struct fsb_mapping *m;

	printf(VERSION "\n\n");
	printf("I need an integer argument: %s <fsb>\n", progname);
	printf("possible values are:\n");
	m = w83194r;
	while (m->fsb != 0) {
		printf("        fsb=%-3i (pci=%i, reg=0x%02x)\n",
				m->fsb, m->pci, m->reg);
		m++;
	}
	exit(1);
}

void dumpsmbus(int file, int addr) {
	int res, reg;
	union i2c_smbus_data buf;

	if (ioctl(file,I2C_SLAVE,addr) < 0) { perror("i2c_slave()"); return; }
	memset(buf.block, 0, sizeof(buf));
	res = i2c_smbus_access(file,I2C_SMBUS_READ,0,I2C_SMBUS_BLOCK_DATA,&buf);
	if (res != 0) { perror("dumpsmbus(): i2c_smbus_access()"); }
	printf("Registers on 0x%02x:", addr);
	for (reg=0;reg<buf.block[0];reg++) {
		printf(" %i=0x%02x",reg,buf.block[1+reg]);
	}
	printf("\n");
}

int main(int argc, char *argv[])
{
	int file;
	int addr = 0x69;	// The I2C address 
	__u8 reg = 0;		// Device register to access 
	int value;
	struct fsb_mapping *m;
	__s32 res;
	int adapter_nr = 0; /* probably dynamically determined */
	char filename[20];
	union i2c_smbus_data buf;

	if (argc>1) {
		value = atoi(argv[1]);
		if (value == 0 && (strcmp(argv[1], "0") != 0)) usage(argv[0]);
	} else value = SMB_QUERY_ONLY;
	if (value > 0) {
		m = w83194r;
		while (m->fsb != 0) {
			if (m->fsb == value) {
				printf("Setting fsb=%i pci=%i (reg=0x%02x)\n",
						m->fsb, m->pci, m->reg);
				break;
			}
			m++;
		}
		if (m->fsb == 0) {
			printf("No entry found for fsb=%i!\n", value);
			//sleep(3);
			exit(23);
		}
	}

	sprintf(filename,"/dev/i2c-%d",adapter_nr);
	if ((file = open(filename,O_RDWR)) < 0) {
		perror("open()");
		exit(1);
	}

	dumpsmbus(file, addr);

	if (value >= 0) {
		// write stuff

		if (value == SMB_RESET) {
			value = 0;
		} else {
			value = m->reg;
			value = 8+((value & 0x7)<<4) + ((value &0x8)>>1);
		}
		printf("Writing 0x%02x to reg 0x00...\n", value);
		//exit(32);

		memset(buf.block, 0, sizeof(buf));
		buf.block[0] = 1;
		buf.block[1] = value;

		res = i2c_smbus_access(file,I2C_SMBUS_WRITE,0,I2C_SMBUS_BLOCK_DATA,&buf);
		if (res != 0) { perror("i2c_smbus_access"); }

		dumpsmbus(file, addr);
	}

	close (file);
}



